/*
 * CSS reset
 * * * * * * * * * * * * * * * * * *
*/

html {
	box-sizing: border-box;
}
*, *:before, *:after {
	box-sizing: inherit;
}
html, body { margin: 0; padding: 0; width: 100%; height: 100%; }
p,h1,h2,h3,h4,h5,h6 { margin: 0; padding: 0; font-weight: 400; }
body {
	font-family: 'Helvetica Neue', 'Helvetica', 'Arial', sans-serif;
	background: #000;
	color: #fff;
	color: rgba(255,255,255,0.8);
	font-size: 10px;
	line-height: 1;
}


/*
 * Elements
 * * * * * * * * * * * * * * * * * *
*/

#disconnected-message {
	display: none;
	position: relative;
	z-index: 800;
}

.disconnected-inner {
	position: fixed;
	z-index: 50;
	top: 50%;
	left: 50%;
	width: 600px;
	height: 80px;
	margin: -40px 0 0 -300px;
	border: 10px solid #a41b0d;
	background: #87190c;
	color: #ecd0cd;
	font-weight: 600;
	font-size: 40px;
	text-align: center;
	line-height: 60px;
}

#disconnected-message:before {
	content: '';
	position: fixed;
	z-index: 5;
	background: rgba(40,0,0,0.5);
	left: 0;
	top: 0;
	width: 100%;
	height: 100%;
}

body.disconnected #disconnected-message {
	display: block;
}

#pulse {
	position: fixed;
	z-index: 1000;
	left: 10px;
	top: 10px;
	background: #16314f;
	width: 10px;
	height: 10px;
	border-radius: 100%;
	display: none;
}
.disconnected #pulse {
	background: #a41b0d;
	width: 20px;
	height: 20px;
}

.loader {
	font-size: 1200%;
	padding: 200px 0 0 0;
	color: rgba(255,255,255,0.7);
	text-align: center;
}
.loader p {
	margin-top: 24px;
	font-size: 24px;
}


/*
 * Nodes
 * * * * * * * * * * * * * * * * * *
*/

#container,
.pipeline,
.nodes-container,
.groups-container {
	width: 100%;
	height: 100%;
}

.nodes-container,
.groups-container {
	padding: 5px 0;
}

.node {
	float: left;
	width: 25%;
	height: 20%;
	padding: 5px;
	transition: opacity 0.5s;
}
.node-inner {
	height: 100%;
	background: rgba(100,100,100,0.25);
	position: relative;
	border-radius: 5px;
	overflow: hidden;
}
.node-content {
	position: relative;
	height: 100%;
	z-index: 50;
	padding: 15px;
	opacity: 1;
	transition: opacity 0.5s;
}

.node-icon {
	float: left;
	font-size: 260%;
	margin: 0;
	color: rgba(0,0,0,0.8);
	background: rgba(255,255,255,0.8);
	text-align: center;
	padding: 6px;
	width: 40px;
	height: 40px;
	border-radius: 100%;
}
.node-name {
	display: block;
	text-decoration: none;
	font-size: 220%;
	line-height: 0.8;
	font-weight: 400;
	color: rgba(255,255,255,0.8);
	margin: 4px 0 0 55px;
}
.node-type {
	display: block;
	font-size: 140%;
	font-weight: 300;
	letter-spacing: 1px;
	color: rgba(255,255,255,0.4);
	margin: 3px 0 0 55px;
}

.node-time {
	position: absolute;
	top: 45%;
	right: 15px;
	text-align: right;
}
.node-time-average {
	font-size: 260%;
}
.node-time-percentile {
	margin: 5px 0 0 0;
	font-size: 180%;
	color: rgba(255,255,255,0.5);
}

.node-waiting {
	position: absolute;
	top: 45%;
	left: 15px;
}
.node-waiting-main {
	font-size: 500%;
	color: rgba(255,255,255,0.8);
}
.node-waiting-alt {
	font-size: 220%;
	font-weight: 300;
	color: rgba(255,255,255,0.4);
}

/* node status */
.node-status-zero .node-content {
	opacity: 0.5;
}
.node-status-zero {
	opacity: 0.7;
}
.node-status-low .node-content {
	opacity: 0.9;
}
.node-status-warning .node-icon {
	background: rgba(255, 222, 34, 0.8);
}
.node-status-warning .node-circle {
	background: rgba(255, 222, 34, 0.8);
}
.node-status-danger .node-icon {
	background: rgba(255, 42, 57, 0.80);
}
.node-status-danger .node-circle {
	background: rgba(255,0,20,1);
}


/*
 * Bar graph
 * * * * * * * * * * * * * * * * * *
*/

.node-graph {
	position: absolute;
	z-index: 1;
	top: 0;
	left: 0;
	right: 0;
	height: 100%;
}
.node-graph-bar {
	display: block;
	float: right;
	width: 10%;
	height: 100%;
}
.node-graph-bar-part {
	display: block;
	color: rgba(255,255,255,0.2);
	text-align: center;
	font-size: 100%;
	line-height: 200%;
	overflow: hidden;
	transition: height 0.5s;
}
.node-graph-success {
	background: rgba(0,255,150,0.1);
}
.node-graph-error {
	background: rgba(255,0,20,0.15);
}
.node-graph-timeout {
	background: rgba(255, 222, 34, 0.15);
}


/*
 * Circle
 * * * * * * * * * * * * * * * * * *
*/

.node-circle {
	position: absolute;
	z-index: 10;
	top: 100%;
	left: 0;
	margin: 0;
	border-radius: 100%;
	background: #fff;
	opacity: 0.15;
	transition: width 0.5s, height 0.5s, margin-left 0.5s, margin-top 0.5s;
}


/*
 * Detail
 * * * * * * * * * * * * * * * * * *
*/

.failed-message {
	font-size: 200%;
	margin: 15px;
	padding: 15px;
	border: 1px solid rgba(255,255,255,0.2);
	border-radius: 5px;
}


/*
 * Group
 * * * * * * * * * * * * * * * * * *
*/


.group{
	overflow: hidden;
	float: left;
	transition: opacity 0.5s;
	position: relative;
	padding: 5px;
}

.group-content {
	background: #212121;
	padding: 5px;
	position: relative;
	height: 100%;
	border: 1px solid #2D2D2D;
	border-radius: 0;
}

.group-content-inner {
	position: relative;
	z-index: 50;
}

.group .node-graph {
	top: 50%;
	height: 50%;
}

.group h1{
	margin-bottom:10px;
	padding-bottom:10px;
	border-bottom:1px solid #2D2D2D;
	font-size:17px;
}

.circle-icon {
	width:25px;
	height:25px;
	margin-right:1%;
	background:#A7A7A7;
	border-radius:40em;
	display: inline-block;
	text-align: center;
	line-height:24px;
	font-size:18px;
	color:#212121;
}

.circle-icon-green {
	background: green;
}


/*
 * Status icons
 * * * * * * * * * * * * * * * * * *
*/

.status-icons{
	list-style:none;
	margin:0px;
	padding:0px;
}
.status-icons li{
	font-size:20px;
	float:left;
}
.status-icons li span{
	float:left;
	margin-right:5px;
}
.fa-check-circle{
	color:#199E19;
}
.fa-exclamation-triangle{
	color:#BF1905;
}
.fa-cog{
	color:rgb(177, 177, 177);
}


/*
 * Status Text
 * * * * * * * * * * * * * * * * * *
*/

.status-text{
	display:block;
	clear:both;
	font-size:16px;
	margin-top:10px;
	float:left;
}

.status-text.warning{
	color:#BF1905;
}

.label{
	font-size:10px;
	clear:both;
	display: block;
	color:#797777;
	margin-bottom:5px;
}


/*
 * Stats
 * * * * * * * * * * * * * * * * * *
*/

.stats{
	width:100%;
	float:left;
	margin-top:10px;
	border-top:1px solid #2D2D2D;
}
.stats .col{
	width:50%;
	float:left;
	font-size:25px;
	padding-top:10px;
	padding-bottom:5px;
	border-right:1px solid #2D2D2D;
	padding-left:10px;
}
.col:last-child{
	border:0px;
}

.small{
	font-size:18px;
	opacity:0.7;
}

.hidden{
	display:none;
}

/*
 * Animations
 * * * * * * * * * * * * * * * * * *
*/


@-webkit-keyframes flash {
  from, 50%, to {
    opacity: 1;
  }

  25%, 75% {
    opacity: 0;
  }
}

@keyframes flash {
  from, 50%, to {
    opacity: 1;
  }

  25%, 75% {
    opacity: 0;
  }
}

.flash{
	-webkit-animation-duration: 1.5s;
	animation-duration: 1.5s;
	-webkit-animation-iteration-count:infinite;
	animation-iteration-count:infinite;
	-webkit-animation-fill-mode: both;
	animation-fill-mode: both;
	-webkit-animation-name: flash;
	animation-name: flash;
}
